# -*- coding: utf-8 -*-
# @Time    : 2024/6/11
# @Author  : 魏志亮
import sys
from os import *

from utils.conf.read_conf import yaml_conf, read_conf


def get_exec_data(batch_no=None, read_type=None, run_count=1):
    if batch_no and read_type:
        data = get_data_by_batch_no_and_type(batch_no, read_type)
        if data is None:
            raise ValueError(f"未找到批次号:{batch_no},类型:{read_type}")

    else:
        data = get_batch_exec_data(run_count)
        if data is None:
            trans_print("当前有任务在执行")
            sys.exit(0)
        elif len(data.keys()) == 0:
            trans_print("当前无任务")
            sys.exit(0)

    return data


def run(batch_no=None, read_type=None, save_db=True, run_count=1):
    update_timeout_trans_data()
    data = get_exec_data(batch_no, read_type, run_count)

    exec_process = None
    if data['transfer_type'] in ['second', 'minute']:
        exec_process = MinSecTrans(data=data, save_db=save_db)

    if data['transfer_type'] in ['fault', 'warn']:
        exec_process = FaultWarnTrans(data=data, save_db=save_db)

    if exec_process is None:
        raise Exception("No exec process")
    exec_process.run()


if __name__ == '__main__':

    env = 'dev'
    if len(sys.argv) >= 2:
        env = sys.argv[1]

    conf_path = path.abspath(f"./conf/etl_config_{env}.yaml")
    environ['ETL_CONF'] = conf_path
    yaml_config = yaml_conf(conf_path)
    environ['env'] = env
    run_count = int(read_conf(yaml_config, "run_batch_count", 1))

    from utils.log.trans_log import trans_print
    from service.plt_service import get_batch_exec_data, get_data_by_batch_no_and_type, update_timeout_trans_data
    from etl.wind_power.fault_warn.FaultWarnTrans import FaultWarnTrans
    from etl.wind_power.min_sec.MinSecTrans import MinSecTrans

    trans_print("所有请求参数:", sys.argv, "env:", env, "最大可执行个数:", run_count)
    trans_print("配置文件路径:", environ.get("ETL_CONF"))

    run(run_count=run_count)
