# -*- coding: utf-8 -*-
"""
Created on Tue Jul  9 16:28:48 2024

@author: Administrator
"""
from os import *

import chardet
import pandas as pd

pd.options.mode.copy_on_write = True


# 获取文件编码
def detect_file_encoding(filename):
    # 读取文件的前1000个字节（足够用于大多数编码检测）
    with open(filename, 'rb') as f:
        rawdata = f.read(1000)
    result = chardet.detect(rawdata)
    encoding = result['encoding']

    if encoding is None:
        encoding = 'gb18030'

    if encoding and encoding.lower() == 'gb2312' or encoding.lower().startswith("windows"):
        encoding = 'gb18030'
    return encoding


# 读取数据到df
def read_file_to_df(file_path, read_cols=list(), header=0):
    df = pd.DataFrame()
    if str(file_path).lower().endswith("csv") or str(file_path).lower().endswith("gz"):
        encoding = detect_file_encoding(file_path)
        end_with_gz = str(file_path).lower().endswith("gz")
        if read_cols:
            if end_with_gz:
                df = pd.read_csv(file_path, encoding=encoding, usecols=read_cols, compression='gzip', header=header)
            else:
                df = pd.read_csv(file_path, encoding=encoding, usecols=read_cols, header=header, on_bad_lines='warn')
        else:

            if end_with_gz:
                df = pd.read_csv(file_path, encoding=encoding, compression='gzip', header=header)
            else:
                df = pd.read_csv(file_path, encoding=encoding, header=header, on_bad_lines='warn')

    else:
        xls = pd.ExcelFile(file_path)
        # 获取所有的sheet名称
        sheet_names = xls.sheet_names
        for sheet in sheet_names:
            if read_cols:
                df = pd.concat([df, pd.read_excel(xls, sheet_name=sheet, header=header, usecols=read_cols)])
            else:
                df = pd.concat([df, pd.read_excel(xls, sheet_name=sheet, header=header)])

    return df


def __build_directory_dict(directory_dict, path, filter_types=None):
    # 遍历目录下的所有项
    for item in listdir(path):
        item_path = path.join(path, item)
        if path.isdir(item_path):
            __build_directory_dict(directory_dict, item_path, filter_types=filter_types)
        elif path.isfile(item_path):
            if path not in directory_dict:
                directory_dict[path] = []

            if filter_types is None or len(filter_types) == 0:
                directory_dict[path].append(item_path)
            elif str(item_path).split(".")[-1] in filter_types:
                if str(item_path).count("~$") == 0:
                    directory_dict[path].append(item_path)

    # 读取所有文件


# 读取路径下所有的excel文件
def read_excel_files(read_path):
    directory_dict = {}
    __build_directory_dict(directory_dict, read_path, filter_types=['xls', 'xlsx', 'csv', 'gz'])

    return [path for paths in directory_dict.values() for path in paths if path]


# 创建路径
def create_file_path(path, is_file_path=False):
    if is_file_path:
        path = path.dirname(path)

    if not path.exists(path):
        makedirs(path, exist_ok=True)


if __name__ == '__main__':
    # path = r'/data/download/大唐玉湖性能分析离线分析/05整理数据/气象站数据'
    # save_path = r'/data/download/大唐玉湖性能分析离线分析/06整理数据/气象站数据'
    path = r'Z:\大唐玉湖性能分析离线分析\05整理数据\气象站数据'
    save_path = r'Z:\大唐玉湖性能分析离线分析\06整理数据\气象站数据'

    fengsu_df = read_file_to_df(path.join(path, '风速.csv'), read_cols=['当前时间', '实际风速'])
    fengxiang_df = read_file_to_df(path.join(path, '风向.csv'), read_cols=['当前时间', '实际风向'])
    fuzhaodu_df = read_file_to_df(path.join(path, '辐照度.csv'), read_cols=['时间', '水平总辐照度', '倾斜总辐照度', '散射辐照度'])
    shidu_df = read_file_to_df(path.join(path, '湿度.csv'), read_cols=['时间', '实际湿度'])
    wendu_df = read_file_to_df(path.join(path, '温度.csv'), read_cols=['时间', '实际温度'])
    yali_df = read_file_to_df(path.join(path, '压力.csv'), read_cols=['时间', '实际气压'])

    fengsu_df.rename(columns={'当前时间': '时间'}, inplace=True)
    fengxiang_df.rename(columns={'当前时间': '时间'}, inplace=True)

    dfs = [fengxiang_df, fengsu_df, fuzhaodu_df, shidu_df, wendu_df, yali_df]

    for df in dfs:
        df['时间'] = pd.to_datetime(df['时间'])
        df.set_index(keys='时间', inplace=True)

    df = pd.concat(dfs, axis=1)
    create_file_path(save_path, is_file_path=False)
    df.to_csv(path.join(save_path, '气象合并.csv'), encoding='utf-8')
